<?php 

namespace Drupal\monarch_bootstrap\Layout;

use Drupal\Core\Layout\LayoutDefault;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\DependencyInjection\DependencySerializationTrait;

class BootstrapContainer extends LayoutDefault implements ContainerFactoryPluginInterface {
  use DependencySerializationTrait;

  /**
   * The module handler service
   */
  protected $moduleHandler;

  /**
   * Constructs a BootstrapContainer object.
   * 
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler service.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, ModuleHandlerInterface $module_handler) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->moduleHandler = $module_handler;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('module_handler')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return parent::defaultConfiguration() + [
      'container' => 'container',
      'bg_color' => '',
      'extra_classes' => '',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $configuration = $this->getConfiguration();
    $form['container'] = [
      '#type' => 'select',
      '#title' => $this->t('Container Style'),
      '#required' => TRUE,
      '#options' => [
        'container' => $this->t('Default'),
        'container-fluid' => $this->t('Fluid'),
        'container-sm' => $this->t('Small'),
        'container-md' => $this->t('Medium'),
        'container-lg' => $this->t('Large'),
        'container-xl' => $this->t('X-Large'),
        'container-xxl' => $this->t('XX-Large'),
      ],
      '#default_value' => $configuration['container'],
    ];

    $color_options = [
      '' => $this->t('None'),
      'bg-primary' => $this->t('Primary'),
      'bg-secondary' => $this->t('Secondary'),
      'bg-light' => $this->t('Light'),
      'bg-dark' => $this->t('Dark'),
    ]; 

    // Let other modules/themes change background colors
    $this->moduleHandler->alter('monarch_bootstrap_bg_color', $color_options);

    $form['bg_color'] = [
      '#type' => 'select',
      '#title' => $this->t('Background Color'),
      '#options' => $color_options,
      '#default_value' => $configuration['bg_color'],
    ];

    $form['extra_classes'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Extra classes'),
      '#default_value' => $configuration['extra_classes'],
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $this->configuration['container'] = $form_state->getValue('container');
    $this->configuration['bg_color'] = $form_state->getValue('bg_color');
    $this->configuration['extra_classes'] = $form_state->getValue('extra_classes');
  }

}